/*
 * Copyright (c) 2006, 2007 by Intevation GmbH
 *
 * Author: Sascha L. Teichmann <teichmann@intevation.de>
 *
 * This program is free software under the LGPL (>=v2.1)
 * Read the file LGPL.txt coming with the software for details.
 */

package de.intevation.osaas;

import java.io.OutputStream;
import java.io.FilterOutputStream;
import java.io.FilterInputStream;
import java.io.IOException;

import java.util.LinkedList;

public class Base64Encoder {
    public static final int LINE_LENGTH = 76;

    protected static final int [] trans = new int[64];

    private static final int MASK = 255 >> 2;

    static {
        for (int i = 0; i < 26; ++i)
            trans[i] = 'A' + i;

        for (int i = 26; i < 52; ++i)
            trans[i] = 'a' + (i-26);

        for (int i = 52; i < 62; ++i)
            trans[i] = '0' + (i-52);

        trans[62] = '+';
        trans[63] = '/';
    }

    private Base64Encoder() {
    }

    public static class InputStream extends FilterInputStream {
        protected int cpl;
        protected int cc;
        protected int state;
        protected int back;

        // XXX: Inefficient!
        protected LinkedList queue;

        public InputStream(java.io.InputStream in) {
            this(in, LINE_LENGTH);
        }

        public InputStream(java.io.InputStream in, int cpl) {
            super(in);
            this.cpl = cpl;
            queue = new LinkedList();
        }

        public int read() throws IOException {
            if (!queueEmpty())
                return out(dequeue());

            int b, b0;

            switch (state) {
                case 0:
                    if ((b = in.read()) < 0) {
                        state = 3;
                        return out(b);
                    }
                    state = 1;
                    back = (b & 3) << 4;
                    return out(trans[(b >> 2) & MASK]);

                case 1:
                    if ((b = in.read()) < 0) {
                        enqueue('=');
                        enqueue('=');
                        state = 3;
                        return out(trans[back]);
                    }
                    state = 2;
                    b0 = back;
                    back = (b & 0xf) << 2;
                    return out(trans[b0 | ((b >> 4) & 0xf)]);

                case 2:
                    if ((b = in.read()) < 0) {
                        state = 3;
                        enqueue('=');
                        return(trans[back]);
                    }
                    state = 0;

                    enqueue(trans[b & MASK]);
                    return out(trans[back | ((b >> 6) & 3)]);

                case 3:
                  state = -1;
                    return cc > 0 ? out('\n') : -1;
            }

            return -1;
        }

        public int read(byte [] buf, int ofs, int len) throws IOException {
            if (state < 0) return -1;
            if (len < 0) return 0;
            int b;
            int start = ofs;
            int end = ofs+len;
            while (ofs < end && (b = read()) != -1)
                buf[ofs++] = (byte)b;
            return ofs-start;
        }

        protected boolean queueEmpty() {
            return queue.isEmpty();
        }

        protected int dequeue() {
            return ((Integer)queue.removeFirst()).intValue();
        }

        protected void enqueue(int b) {
            queue.addLast(new Integer(b));
        }

        protected int out(int b) {
            if (cc++ >= cpl) {
                cc = 0;
                enqueue(b);
                return '\n';
            }
            return b;
        }

    } // class InputStream

    public static class OutputStream extends FilterOutputStream {
        protected int state;
        protected int back;
        protected int cpl;
        protected int cc;

        public OutputStream(java.io.OutputStream parent) {
            this(parent, LINE_LENGTH);
        }

        public OutputStream(java.io.OutputStream parent, int cpl) {
            super(parent);
            this.cpl = cpl;
        }

        protected final void out(int b) throws IOException {
            out.write(b);
            if (++cc >= cpl) {
                cc = 0;
                out.write('\n');
            }
        }

        public void write(int b) throws IOException {
            switch (state) {
                case 0:
                    back = (b & 3) << 4;
                    out(trans[(b >> 2) & MASK]);
                    state = 1;
                    break;

                case 1:
                    int nback = (b & 0xf) << 2;
                    out(trans[back | ((b >> 4) & 0xf)]);
                    back = nback;
                    state = 2;
                    break;

                case 2:
                    out(trans[back | ((b >> 6) & 3)]);
                    out(trans[b & MASK]);
                    state = 0;
                    break;
            }
        }

        public void flush() throws IOException {
            out.flush();
        }

        public void close() throws IOException {
            switch (state) {
                case -2:
                case -1:
                    return;

                case 1:
                    out(trans[back]);
                    out('=');
                    out('=');
                    break;

                case 2:
                    out(trans[back]);
                    out('=');
                    break;
            }
            if (cc > 0) {
                cc = 0;
                out.write('\n');
            }
            state = -2;
            flush();
            out.close();
        }
    } // class OutputStream
}
// end of file
