# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

import unittest

from support import FileTestMixin
from osaas.config import OSAASOption, ConfigXMLPathError
from osaas.run import OSAASServerProgram

class TestOSAASOption(unittest.TestCase):

    def test_xml_path(self):
        opt = OSAASOption("--port", type="int", xml_path="OSAASConfig/Port")
        self.assertEquals(opt.xml_path, "OSAASConfig/Port")



class ConfigParserTest(unittest.TestCase, FileTestMixin):

    config = ""

    def setUp(self):
        self.filename = self.create_temp_file(self.id() + ".cfg", self.config)


class TestConfigParsing(ConfigParserTest):

    config = """\
<?xml version='1.0'?>
<OSAASConfig>
  <Port>6688</Port>
  <AccessLog>/var/log/osaas-access.log</AccessLog>
  <ErrorLog>/var/log/osaas-error.log</ErrorLog>
</OSAASConfig>
"""

    def test(self):
        prog = OSAASServerProgram()
        opts, rest = prog.parse_options(["--config-file=%s" % self.filename])
        self.assertEquals(opts.port, 6688)
        self.assertEquals(opts.access_log, "/var/log/osaas-access.log")
        self.assertEquals(opts.error_log, "/var/log/osaas-error.log")


class TestConfigParsing_MissingEntries(ConfigParserTest):

    config = """\
<?xml version='1.0'?>
<OSAASConfig>
  <ErrorLog>/var/log/osaas-error.log</ErrorLog>
</OSAASConfig>
"""

    def test(self):
        prog = OSAASServerProgram()
        opts, rest = prog.parse_options(["--config-file=%s" % self.filename])
        self.assertEquals(opts.port, 8989)
        self.assertEquals(opts.access_log, None)
        self.assertEquals(opts.error_log, "/var/log/osaas-error.log")


class TestConfigParsing_AmbiguousEntries(ConfigParserTest):

    config = """\
<?xml version='1.0'?>
<OSAASConfig>
  <ErrorLog>/var/log/osaas-error.log</ErrorLog>
  <ErrorLog>/var/log/osaas-error2.log</ErrorLog>
</OSAASConfig>
"""

    def test(self):
        prog = OSAASServerProgram()
        self.assertRaises(ConfigXMLPathError,
                          prog.parse_options,
                          ["--config-file=%s" % self.filename])

class TestConfigParsing_DBConnection(ConfigParserTest):

    config = """\
<?xml version='1.0'?>
<OSAASConfig>
  <ErrorLog>/var/log/osaas-error.log</ErrorLog>
  <DBConnection>
    <User>owsdbuser</User>
    <Password>secret</Password>
    <DSN>mydsn</DSN>
  </DBConnection>
</OSAASConfig>
"""

    def test(self):
        prog = OSAASServerProgram()
        opts, rest = prog.parse_options(["--config-file=%s" % self.filename])
        self.assertEquals(opts.db_user, "owsdbuser")
        self.assertEquals(opts.db_password, "secret")
        self.assertEquals(opts.db_dsn, "mydsn")


class TestConfigParsing_DBRules(ConfigParserTest):

    config = """\
<?xml version='1.0'?>
<OSAASConfig>
  <ErrorLog>/var/log/osaas-error.log</ErrorLog>
  <DBRules>
    <insert table='MyOWSAccounting'>
      <value param='SERVICE' column='service' type='character varying(10)'/>
      <value param='REQUEST' column='req' type='character varying(10)'/>
      <value param='BBOX' column='bbox' type='character varying(100)'/>
      <value param='WIDTH' column='width' type='int'/>
      <value param='HEIGHT' column='height' type='int'/>
    </insert>
  </DBRules>
</OSAASConfig>
"""

    def test(self):
        prog = OSAASServerProgram()
        opts, rest = prog.parse_options(["--config-file=%s" % self.filename])
        rules = opts.db_rules
        self.assertEquals(len(rules), 1)
        rule = rules[0]
        # in addition to checking the values, we also check the types
        # because cx_Oracle is very picky and does not accept unicode
        # objects as statements even when they only contain ascii
        # characters
        self.assertEquals(rule.table, "MyOWSAccounting")
        self.failUnless(isinstance(rule.table, str))
        self.assertEquals([(f.parameter, f.column, f.column_type)
                           for f in rule.fields],
                          [("SERVICE", "service", "character varying(10)"),
                           ("REQUEST", "req", "character varying(10)"),
                           ("BBOX", "bbox", "character varying(100)"),
                           ("WIDTH", "width", "int"),
                           ("HEIGHT", "height", "int")])
        self.assertEquals([(type(f.parameter), type(f.column),
                            type(f.column_type))
                           for f in rule.fields],
                          [(str, str, str)] * 5)
