# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Tests for osaas.formparser"""

import sys
import datetime
import unittest
import traceback

import support

from osaas.formparser import parse_formdata, FormParserException


class FormParsingTests(unittest.TestCase, support.AttributeTestMixin):

    def test_convert_form_urlencoded(self):
        # In the test data in this test case, the colons are
        # percent-escaped as well, because Java does it that way.
        formdata = "&".join(["user=karl",
                             "responsetime=2007-06-01T12%3A14%3A16Z",
                             "wmsidextern=example.com%2Fcgi-bin%2Ffridawms",
                             "wmsidintern=localhost%2Fcgi-bin%2Fmyfrida",
                             "requeststring=VERSION=1.1.1%26REQUEST=GetMap"
                             "%26FORMAT=image%2Fpng%26TRANSPARENT=TRUE"
                             "%26WIDTH=460%26HEIGHT=348"
                             "%26EXCEPTIONS=application%2Fvnd.ogc.se_xml"
                             "%26BGCOLOR=0xffffff%26BBOX=0.0,0.0,460.0,348.0"
                             "%26LAYERS=gewaesser%26STYLES=%26SRS=EPSG%3A4326"
                             "%26SERVICE=WMS"])
        self.check_attributes(parse_formdata(formdata),
                              user="karl",
                              responsetime=datetime.datetime(2007, 6, 1,
                                                             12, 14, 16),
                              wmsidextern="example.com/cgi-bin/fridawms",
                              wmsidintern="localhost/cgi-bin/myfrida",
                              requeststring=("VERSION=1.1.1&REQUEST=GetMap"
                                             "&FORMAT=image/png"
                                             "&TRANSPARENT=TRUE"
                                             "&WIDTH=460&HEIGHT=348"
                                      "&EXCEPTIONS=application/vnd.ogc.se_xml"
                                             "&BGCOLOR=0xffffff"
                                             "&BBOX=0.0,0.0,460.0,348.0"
                                             "&LAYERS=gewaesser&STYLES="
                                             "&SRS=EPSG:4326"
                                             "&SERVICE=WMS"),
                              VERSION="1.1.1",
                              REQUEST="GetMap",
                              FORMAT="image/png",
                              TRANSPARENT="TRUE",
                              WIDTH="460",
                              HEIGHT="348",
                              EXCEPTIONS="application/vnd.ogc.se_xml",
                              BGCOLOR="0xffffff",
                              BBOX="0.0,0.0,460.0,348.0",
                              LAYERS="gewaesser",
                              STYLES="",
                              SRS="EPSG:4326",
                              SERVICE="WMS")

    def check_exception(self, exception_class, exception_message,
                        function, *args, **kw):
        try:
            function(*args, **kw)
        except exception_class, err:
            self.assertEquals(str(err), exception_message)
        except:
            self.fail("Unexpected exception:\n%s"
                      % "".join(traceback.format_exception(*sys.exc_info())))
        else:
            self.fail("Calling %r with %r and %r did not raise an exception"
                      % (function, args, kw))


    def test_missing_required_parameters_in_formdata(self):
        # formdata is missing the required field "user"
        formdata = "&".join(["responsetime=2007-06-01T12:14:16Z",
                             "wmsidextern=example.com%2Fcgi-bin%2Ffridawms",
                             "wmsidintern=localhost%2Fcgi-bin%2Fmyfrida",
                             "requeststring=VERSION=1.1.1%26REQUEST=GetMap"
                             "%26FORMAT=image%2Fpng%26TRANSPARENT=TRUE"
                             "%26WIDTH=460%26HEIGHT=348"
                             "%26EXCEPTIONS=application%2Fvnd.ogc.se_xml"
                             "%26BGCOLOR=0xffffff%26BBOX=0.0,0.0,460.0,348.0"
                             "%26LAYERS=gewaesser%26STYLES=%26SRS=EPSG:4326"
                             "%26SERVICE=WMS"])
        record = parse_formdata(formdata)
        self.check_exception(FormParserException,
                             "formdata is missing required field 'user'",
                             record.validate_fields)

    def test_missing_required_parameters_in_requeststring(self):
        # requeststring is missing the required field "REQUEST"
        formdata = "&".join(["user=karl", "responsetime=2007-06-01T12:14:16Z",
                             "wmsidextern=example.com%2Fcgi-bin%2Ffridawms",
                             "wmsidintern=localhost%2Fcgi-bin%2Fmyfrida",
                             "requeststring=VERSION=1.1.1"
                             "%26FORMAT=image%2Fpng%26TRANSPARENT=TRUE"
                             "%26WIDTH=460%26HEIGHT=348"
                             "%26EXCEPTIONS=application%2Fvnd.ogc.se_xml"
                             "%26BGCOLOR=0xffffff%26BBOX=0.0,0.0,460.0,348.0"
                             "%26LAYERS=gewaesser%26STYLES=%26SRS=EPSG:4326"
                             "%26SERVICE=WMS"])
        record = parse_formdata(formdata)
        self.check_exception(FormParserException,
                             "formdata is missing required field 'REQUEST'",
                             record.validate_fields)

    def test_missing_optional_parameters(self):
        # requeststring is missing the optional field "SERVICE"
        formdata = "&".join(["user=karl", "responsetime=2007-06-01T12:14:16Z",
                             "wmsidextern=example.com%2Fcgi-bin%2Ffridawms",
                             "wmsidintern=localhost%2Fcgi-bin%2Fmyfrida",
                             "requeststring=VERSION=1.1.1%26REQUEST=GetMap"
                             "%26FORMAT=image%2Fpng%26TRANSPARENT=TRUE"
                             "%26WIDTH=460%26HEIGHT=348"
                             "%26EXCEPTIONS=application%2Fvnd.ogc.se_xml"
                             "%26BGCOLOR=0xffffff%26BBOX=0.0,0.0,460.0,348.0"
                             "%26LAYERS=gewaesser%26STYLES=%26SRS=EPSG:4326"])
        record = parse_formdata(formdata)
        self.check_attributes(parse_formdata(formdata),
                              user="karl",
                              responsetime=datetime.datetime(2007, 6, 1,
                                                             12, 14, 16),
                              wmsidextern="example.com/cgi-bin/fridawms",
                              wmsidintern="localhost/cgi-bin/myfrida",
                              requeststring=("VERSION=1.1.1&REQUEST=GetMap"
                                             "&FORMAT=image/png"
                                             "&TRANSPARENT=TRUE"
                                             "&WIDTH=460&HEIGHT=348"
                                      "&EXCEPTIONS=application/vnd.ogc.se_xml"
                                             "&BGCOLOR=0xffffff"
                                             "&BBOX=0.0,0.0,460.0,348.0"
                                             "&LAYERS=gewaesser&STYLES="
                                             "&SRS=EPSG:4326"),
                              VERSION="1.1.1",
                              REQUEST="GetMap",
                              FORMAT="image/png",
                              TRANSPARENT="TRUE",
                              WIDTH="460",
                              HEIGHT="348",
                              EXCEPTIONS="application/vnd.ogc.se_xml",
                              BGCOLOR="0xffffff",
                              BBOX="0.0,0.0,460.0,348.0",
                              LAYERS="gewaesser",
                              STYLES="",
                              SRS="EPSG:4326",
                              SERVICE=None)

    def test_duplicate_parameters(self):
        # The responsetime occurs twice
        formdata = "&".join(["user=karl", "responsetime=2007-06-01T12:14:16Z",
                             "wmsidextern=example.com%2Fcgi-bin%2Ffridawms",
                             "wmsidintern=localhost%2Fcgi-bin%2Fmyfrida",
                             "requeststring=VERSION=1.1.1"
                             "%26FORMAT=image%2Fpng%26TRANSPARENT=TRUE"
                             "%26WIDTH=460%26HEIGHT=348%26REQUEST=GetMap"
                             "%26EXCEPTIONS=application%2Fvnd.ogc.se_xml"
                             "%26BGCOLOR=0xffffff%26BBOX=0.0,0.0,460.0,348.0"
                             "%26LAYERS=gewaesser%26STYLES=%26SRS=EPSG:4326"
                             "%26SERVICE=WMS",
                             "responsetime=2007-06-01T12:14:16Z"])
        self.check_exception(FormParserException,
                             "only one value allowed for field 'responsetime'",
                             parse_formdata, formdata)

    def test_duplicate_parameters_in_requeststring(self):
        # The SERVICE field occurs twice in the requeststring
        formdata = "&".join(["user=karl", "responsetime=2007-06-01T12:14:16Z",
                             "wmsidextern=example.com%2Fcgi-bin%2Ffridawms",
                             "wmsidintern=localhost%2Fcgi-bin%2Fmyfrida",
                             "requeststring=VERSION=1.1.1%26REQUEST=GetMap"
                             "%26FORMAT=image%2Fpng%26TRANSPARENT=TRUE"
                             "%26WIDTH=460%26HEIGHT=348%26SERVICE=WMS"
                             "%26EXCEPTIONS=application%2Fvnd.ogc.se_xml"
                             "%26BGCOLOR=0xffffff%26BBOX=0.0,0.0,460.0,348.0"
                             "%26LAYERS=gewaesser%26STYLES=%26SRS=EPSG:4326"
                             "%26SERVICE=WMS"])
        self.check_exception(FormParserException,
                             "only one value allowed for field 'SERVICE'",
                             parse_formdata, formdata)

    def test_invalid_formdata_syntax_no_equals(self):
        # No equals sign in form field
        formdata = "user"
        self.check_exception(FormParserException,
                             "Error parsing form data: bad query field: 'user'",
                             parse_formdata, formdata)
