# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Tests for UserDB"""

import sys
import os
import unittest
import traceback

from support import FileTestMixin

from osaas.userdb import UserDB, encode_ssha, check_password


class TestSSHA1(unittest.TestCase, FileTestMixin):

    def test_encode_ssha(self):
        self.assertEquals(encode_ssha("secret", "salz"),
                          "{SSHA}uGAUfJr9yps6jrPoJXwpSFb+ndxzYWx6")

    def test_check_password_ssha_correct_password(self):
        self.failUnless(check_password("secret",
                                     "{SSHA}uGAUfJr9yps6jrPoJXwpSFb+ndxzYWx6"))

    def test_check_password_ssha_wrong_password(self):
        self.failIf(check_password("secret",
                                   "{SSHA}ki+V0/wTlQFLv2lL1RShgx5O1EFzYWx6"))

class TestUserDB(unittest.TestCase):

    def test_valid_password(self):
        db = UserDB([("user", "geheim")])
        self.failUnless(db.check_credentials("user", "geheim"))

    def test_invalid_password(self):
        db = UserDB([("user", "geheim")])
        self.failIf(db.check_credentials("user", "geraten"))

    def test_unknown_user(self):
        db = UserDB([("user", "geheim")])
        self.failIf(db.check_credentials("irgendwer", "geraten"))

    def test_add_user(self):
        db = UserDB([])
        self.failIf(db.check_credentials("arthur", "42"))
        db.add_user("arthur", "42")
        self.failUnless(db.check_credentials("arthur", "42"))


class TestUserDBFileIO(unittest.TestCase, FileTestMixin):

    contents = """\
hans:{SSHA}l7c79AuF2IRrdNUBBCDH6Xs3adpsaW51eA==
karl:{SSHA}uGAUfJr9yps6jrPoJXwpSFb+ndxzYWx6
"""

    def setUp(self):
        self.filename = self.create_temp_file(self.id(), self.contents)
        self.userdb = UserDB()
        self.userdb.read(self.filename)

    def test_reading(self):
        self.failUnless(self.userdb.check_credentials("hans", "tux"))
        self.failUnless(self.userdb.check_credentials("karl", "secret"))

    def test_reading_and_writing(self):
        filename = self.temp_file_name(self.id() + "-write")
        self.userdb.write(filename)
        self.assertEquals(open(filename).read(), self.contents)

    def test_nonexisting_file(self):
        os.remove(self.filename)
        db = UserDB()
        try:
            db.read(self.filename)
        except:
            self.fail("Error when opening non-existing file:\n%s"
                      % "".join(traceback.format_exception(*sys.exc_info())))
